@@ -26,16 +26,22 @@
 #include <geode/DataOutput.hpp>
 #include <geode/SystemProperties.hpp>
 
+#include "CacheImpl.hpp"
 #include "ClientConnectionRequest.hpp"
 #include "ClientConnectionResponse.hpp"
 #include "ClientReplacementRequest.hpp"
+#include "FunctionMacros.hpp"
 #include "LocatorListRequest.hpp"
 #include "LocatorListResponse.hpp"
 #include "QueueConnectionRequest.hpp"
 #include "QueueConnectionResponse.hpp"
+#include "TcpConn.hpp"
 #include "TcpSslConn.hpp"
 #include "TcrConnectionManager.hpp"
 #include "ThinClientPoolDM.hpp"
+#include "Version.hpp"
+
+INIT_GNFN("ThinClientLocatorHelper")
 
 namespace apache {
 namespace geode {
@@ -44,17 +50,12 @@ namespace client {
 const size_t BUFF_SIZE = 3000;
 const size_t DEFAULT_CONNECTION_RETRIES = 3;
 
-ThinClientLocatorHelper::ConnectionWrapper::~ConnectionWrapper() {
-  if (conn_ != nullptr) {
-    LOGDEBUG("Closing the locator connection");
-    conn_->close();
-    delete conn_;
-  }
-}
-
 ThinClientLocatorHelper::ThinClientLocatorHelper(
     const std::vector<std::string>& locators, const ThinClientPoolDM* poolDM)
-    : locators_(locators.begin(), locators.end()), m_poolDM(poolDM) {}
+    : locators_(locators.begin(), locators.end()),
+      m_poolDM(poolDM),
+      m_sniProxyHost(""),
+      m_sniProxyPort(0) {}
 
 ThinClientLocatorHelper::ThinClientLocatorHelper(
     const std::vector<std::string>& locators, const std::string& sniProxyHost,
@@ -81,19 +82,17 @@ std::vector<ServerLocation> ThinClientLocatorHelper::getLocators() const {
   }
 
   RandGen randGen;
-  std::random_shuffle(locators.begin(), locators.end(), randGen);
+  std::shuffle(locators.begin(), locators.end(), randGen);
   return locators;
 }
 
-ThinClientLocatorHelper::ConnectionWrapper
-ThinClientLocatorHelper::createConnection(
+std::unique_ptr<Connector> ThinClientLocatorHelper::createConnection(
     const ServerLocation& location) const {
   auto& sys_prop = m_poolDM->getConnectionManager()
                        .getCacheImpl()
                        ->getDistributedSystem()
                        .getSystemProperties();
 
-  Connector* conn;
   const auto port = location.getPort();
   auto timeout = sys_prop.connectTimeout();
   const auto& hostname = location.getServerName();
@@ -101,25 +100,23 @@ ThinClientLocatorHelper::createConnection(
 
   if (sys_prop.sslEnabled()) {
     if (m_sniProxyHost.empty()) {
-      conn = new TcpSslConn(hostname, static_cast<uint16_t>(port), timeout,
-                            buffer_size, sys_prop.sslTrustStore(),
-                            sys_prop.sslKeyStore(),
-                            sys_prop.sslKeystorePassword());
+      return std::unique_ptr<Connector>(new TcpSslConn(
+          hostname, static_cast<uint16_t>(port), timeout, buffer_size,
+          sys_prop.sslTrustStore(), sys_prop.sslKeyStore(),
+          sys_prop.sslKeystorePassword()));
     } else {
-      conn = new TcpSslConn(hostname, timeout, buffer_size, m_sniProxyHost,
-                            m_sniProxyPort, sys_prop.sslTrustStore(),
-                            sys_prop.sslKeyStore(),
-                            sys_prop.sslKeystorePassword());
+      return std::unique_ptr<Connector>(new TcpSslConn(
+          hostname, static_cast<uint16_t>(port), m_sniProxyHost, m_sniProxyPort,
+          timeout, buffer_size, sys_prop.sslTrustStore(),
+          sys_prop.sslKeyStore(), sys_prop.sslKeystorePassword()));
     }
   } else {
-    conn = new TcpConn(hostname, port, timeout, buffer_size);
+    return std::unique_ptr<Connector>(new TcpConn(
+        hostname, static_cast<uint16_t>(port), timeout, buffer_size));
   }
-
-  ConnectionWrapper cw{conn};
-  cw->init();
-  return cw;
 }
 
+static constexpr int32_t kGossipVersion = 1002;
 std::shared_ptr<Serializable> ThinClientLocatorHelper::sendRequest(
     const ServerLocation& location,
     const std::shared_ptr<Serializable>& request) const {
@@ -132,8 +129,14 @@ std::shared_ptr<Serializable> ThinClientLocatorHelper::sendRequest(
     auto conn = createConnection(location);
     auto data =
         m_poolDM->getConnectionManager().getCacheImpl()->createDataOutput();
-    data.writeInt(static_cast<int32_t>(1001));  // GOSSIPVERSION
+    data.writeInt(kGossipVersion);
+    data.writeInt(Version::current().getOrdinal());
     data.writeObject(request);
+    LOGDEBUG(
+        "%s(%p): sending %d bytes to locator: %s", __GNFN__, this,
+        data.getBufferLength(),
+        Utils::convertBytesToString(data.getBuffer(), data.getBufferLength())
+            .c_str());
     auto sentLength = conn->send(
         reinterpret_cast<char*>(const_cast<uint8_t*>(data.getBuffer())),
         data.getBufferLength(), m_poolDM->getReadTimeout());
@@ -141,17 +144,24 @@ std::shared_ptr<Serializable> ThinClientLocatorHelper::sendRequest(
       return nullptr;
     }
     char buff[BUFF_SIZE];
-    auto receivedLength =
-        conn->receive(buff, BUFF_SIZE, m_poolDM->getReadTimeout());
-    if (receivedLength <= 0) {
+    const auto receivedLength = conn->receive(buff, m_poolDM->getReadTimeout());
+    if (!receivedLength) {
       return nullptr;
     }
 
+    LOGDEBUG("%s(%p): received %d bytes from locator: %s", __GNFN__, this,
+             receivedLength,
+             Utils::convertBytesToString(reinterpret_cast<uint8_t*>(buff),
+                                         receivedLength)
+                 .c_str());
+
     auto di = m_poolDM->getConnectionManager().getCacheImpl()->createDataInput(
         reinterpret_cast<uint8_t*>(buff), receivedLength);
 
     if (di.read() == REPLY_SSL_ENABLED && !sys_prop.sslEnabled()) {
-      LOGERROR("SSL is enabled on locator, enable SSL in client as well");
+      LOGERROR(
+          "%s(%p): SSL is enabled on locator %s, enable SSL in client as well",
+          __GNFN__, this, location.toString().c_str());
       throw AuthenticationRequiredException(
           "SSL is enabled on locator, enable SSL in client as well");
     }
@@ -161,8 +171,10 @@ std::shared_ptr<Serializable> ThinClientLocatorHelper::sendRequest(
   } catch (const AuthenticationRequiredException& excp) {
     throw excp;
   } catch (const Exception& excp) {
-    LOGFINE("Exception while querying locator: %s: %s", excp.getName().c_str(),
-            excp.what());
+    LOGFINE("%s(%p): Exception while querying locator: %s: %s", __GNFN__, this,
+            excp.getName().c_str(), excp.what());
+  } catch (...) {
+    LOGFINE("%s(%p): Exception while querying locator", __GNFN__, this);
   }
 
   return nullptr;
@@ -172,8 +184,8 @@ GfErrType ThinClientLocatorHelper::getAllServers(
     std::vector<std::shared_ptr<ServerLocation> >& servers,
     const std::string& serverGrp) const {
   for (const auto& loc : getLocators()) {
-    LOGDEBUG("getAllServers getting servers from server = %s ",
-             loc.getServerName().c_str());
+    LOGDEBUG("%s(%p): getAllServers getting servers from server = %s ",
+             __GNFN__, this, loc.getServerName().c_str());
 
     auto request = std::make_shared<GetAllServersRequest>(serverGrp);
     auto response = std::dynamic_pointer_cast<GetAllServersResponse>(
@@ -197,10 +209,7 @@ GfErrType ThinClientLocatorHelper::getEndpointForNewCallBackConn(
   auto locatorsSize = locators.size();
   auto maxAttempts = getConnRetries();
 
-  LOGFINER(
-      "ThinClientLocatorHelper::getEndpointForNewCallBackConn maxAttempts = "
-      "%zu",
-      maxAttempts);
+  LOGFINER("%s(%p): maxAttempts = %zu", __GNFN__, this, maxAttempts);
 
   for (auto attempt = 0ULL; attempt < maxAttempts;) {
     const auto& loc = locators[attempt++ % locatorsSize];
@@ -232,8 +241,9 @@ GfErrType ThinClientLocatorHelper::getEndpointForNewFwdConn(
   auto maxAttempts = getConnRetries();
 
   LOGFINER(
-      "ThinClientLocatorHelper::getEndpointForNewFwdConn maxAttempts = %zu",
-      maxAttempts);
+      "%s(%p): ThinClientLocatorHelper::getEndpointForNewFwdConn maxAttempts = "
+      "%zu",
+      __GNFN__, this, maxAttempts);
 
   for (auto attempt = 0ULL; attempt < maxAttempts;) {
     const auto& loc = locators[attempt++ % locatorsSize];
@@ -242,11 +252,12 @@ GfErrType ThinClientLocatorHelper::getEndpointForNewFwdConn(
 
     std::shared_ptr<Serializable> request;
     if (currentServer == nullptr) {
-      LOGDEBUG("Creating ClientConnectionRequest");
+      LOGDEBUG("%s(%p): Creating ClientConnectionRequest", __GNFN__, this);
       request =
           std::make_shared<ClientConnectionRequest>(exclEndPts, serverGrp);
     } else {
-      LOGDEBUG("Creating ClientReplacementRequest for connection: %s",
+      LOGDEBUG("%s(%p): Creating ClientReplacementRequest for connection: %s",
+               __GNFN__, this,
                currentServer->getEndpointObject()->name().c_str());
       request = std::make_shared<ClientReplacementRequest>(
           currentServer->getEndpointObject()->name(), exclEndPts, serverGrp);
@@ -260,14 +271,14 @@ GfErrType ThinClientLocatorHelper::getEndpointForNewFwdConn(
 
     response->printInfo();
     if (!response->serverFound()) {
-      LOGFINE("Server not found");
+      LOGFINE("%s(%p): Server not found", __GNFN__, this);
       locatorFound = true;
       continue;
     }
 
     outEndpoint = response->getServerLocation();
-    LOGFINE("Server found at [%s:%d]", outEndpoint.getServerName().c_str(),
-            outEndpoint.getPort());
+    LOGFINE("%s(%p): Server found at [%s:%d]", __GNFN__, this,
+            outEndpoint.getServerName().c_str(), outEndpoint.getPort());
 
     return GF_NOERR;
   }
@@ -283,8 +294,10 @@ GfErrType ThinClientLocatorHelper::updateLocators(
     const std::string& serverGrp) {
   auto locators = getLocators();
   for (const auto& loc : locators) {
-    LOGFINER("Querying locator list at: [%s:%d] for update from group [%s]",
-             loc.getServerName().c_str(), loc.getPort(), serverGrp.c_str());
+    LOGFINER(
+        "%s(%p): Querying locator list at: [%s:%d] for update from group [%s]",
+        __GNFN__, this, loc.getServerName().c_str(), loc.getPort(),
+        serverGrp.c_str());
 
     auto request = std::make_shared<LocatorListRequest>(serverGrp);
     auto response = std::dynamic_pointer_cast<LocatorListResponse>(
